#import packages
import dash
from dash import dcc, Dash, dcc, html, Input, Output, State, dash_table, callback
import json
import matplotlib.pyplot as plt
import importlib
import sys
sys.path.append('stl')
from stlcg import stlcg
from stlcg.stlcg import convert_to_input_values
import torch
import stlcg as viz
import numpy as np
from stlcg.stlcg import Expression
import plotly.express as px
import plotly.graph_objects as go
import pandas as pd

sys.path.append('bayes_opt')
from bayes_opt.bayesian_optimization import BayesianOptimization
from bayes_opt.domain_reduction import SequentialDomainReductionTransformer
import matplotlib.pyplot as plt

from sklearn.svm import SVC
from sklearn.datasets import load_breast_cancer
from sklearn.preprocessing import MinMaxScaler
from sklearn.model_selection import train_test_split
from sklearn.metrics import roc_auc_score
from bayes_opt import UtilityFunction
import warnings
warnings.filterwarnings("ignore")

import fmpy
from fmpy import *

#functions
def sort_start_values(start_values):
    allkeys = list(start_values.keys())
    values = list(start_values.values())

    keys = []
    for i in allkeys:
        s, m, e = i.partition('[')
        keys.append(s)

    alle = keys
    keys = list(dict.fromkeys(keys))

    particular_value = keys[0]
    start_key = []
    start_value = []
    temp_key = []
    temp_value = []

    for k in range(0, len(keys)): 
        particular_value = keys[k] 
    
        for i in range(0, len(allkeys)):
            if particular_value == alle[i]:
                temp_value.append(values[i])
                temp_key.append(allkeys[i])
            else:
                k+1
        start_key.append(temp_key)
        start_value.append(temp_value)
        temp_key = []        
        temp_value = [] 
              
    return start_key, start_value, keys

def make_dict(keys, start_value):
    d2 = dict()
    for i in range(0, len(keys)):
        d1 = dict(zip(keys, start_value))
        d2.update(d1)
    return d2

def multiply_by_constant(d, navn, konst):
    verdi = d.get(navn)
    verdi_ny = [element * konst for element in verdi]
    d.update({navn: verdi_ny})
    return d

def make_start_values(d, start_key):
    d2 = dict()
    start_value = list(d.values())
    for i in range(0, len(start_key)):
        d1 = dict(zip(start_key[i], start_value[i]))
        d2.update(d1)
    return d2

def calculate_robustness(dataframe, robustoutput, min, max):
    
    input = torch.tensor(dataframe[robustoutput], dtype = torch.float, requires_grad=False)
    input = torch.reshape(input, (1, len(input), 1))

    val_max = torch.tensor(max, dtype=torch.float, requires_grad=True)
    val_min = torch.tensor(min, dtype=torch.float, requires_grad=True)

    ϕ1 = stlcg.LessThan(lhs=robustoutput, val=val_max)
    ϕ2 = stlcg.GreaterThan(lhs=robustoutput, val=val_min)
    ϕ1 = stlcg.Always(subformula=ϕ1, interval = [0,2])
    ϕ2 = stlcg.Always(subformula=ϕ2, interval = [0,2])
    ψ = stlcg.And(ϕ1, ϕ2) 
    inputs = (input, input)
    pscale = 1     # "pscale" is the scale used for evaluting predicates
    scale = -1     # "scale" is the scale used in the maxish/minish function. <0 defaults to the true min/max
    rob = ψ.robustness_trace(inputs, pscale=pscale, scale=scale)

    rob2 = rob.detach().numpy()
    robreverse = rob2[0,:]
    dataframe["Robustness"] = robreverse
    minval = dataframe["Robustness"].min()
    
    return dataframe, minval


def plot_result_plotly(dataframe, fontsize):

    fig1 = go.Figure()
    fig1.add_trace(go.Line(x=dataframe["time"], y=dataframe["V_terminal_Cell1"],  name = "V terminal Cell1", line=dict(color='purple')))
    fig1.add_trace(go.Line(x=dataframe["time"], y=dataframe["V_terminal_Cell2"],  name = "V terminal Cell2", line=dict(color='hotpink')))
    fig1.add_trace(go.Line(x=dataframe["time"], y=dataframe["V_terminal_Cell3"],  name = "V terminal Cell3", line=dict(color='darkblue')))
    fig1.add_trace(go.Line(x=dataframe["time"], y=dataframe["V_terminal_Cell4"],  name = "V terminal Cell4", ))
    fig1.update_layout(
        font=dict(
        family="Times New Roman",
        size=fontsize,   
        color = 'black'),
        title = 'Voltage of cells',  
        )  
    fig1.update_xaxes(title_text="Time [sec] ")
    fig1.update_yaxes(title_text="Voltage [V]")
    fig1.show()

    

    fig3 = go.Figure()
    fig3.add_trace(go.Line(x=dataframe["time"], y=dataframe["I_Cell1"],  name = "I Cell1", line=dict(color='purple')))
    fig3.add_trace(go.Line(x=dataframe["time"], y=dataframe["I_Cell2"] , name = "I Cell2", line=dict(color='hotpink')))
    fig3.add_trace(go.Line(x=dataframe["time"], y=dataframe["I_Cell3"], name = "I Cell3", line=dict(color='darkblue')))
    fig3.add_trace(go.Line(x=dataframe["time"], y=dataframe["I_Cell4"], name = "I Cell4"))
    fig3.update_layout(font=dict(
        family="Times New Roman",
        size=fontsize,  
        color = 'black'),
        title = 'Current of cells',   
    )
    fig3.update_xaxes(title_text="Time [sec] ")
    fig3.update_yaxes(title_text="Ampere [A]")
    fig3.show()

    fig4 = go.Figure()
    fig4.add_trace(go.Line(x=dataframe["time"], y=dataframe["SOC_Cell1"], name = "SOC Cell1", line=dict(color='purple')))
    fig4.add_trace(go.Line(x=dataframe["time"], y=dataframe["SOC_Cell2"], name = "SOC Cell2", line=dict(color='hotpink')))
    fig4.add_trace(go.Line(x=dataframe["time"], y=dataframe["SOC_Cell3"], name = "SOC Cell3", line=dict(color='darkblue')))
    fig4.add_trace(go.Line(x=dataframe["time"], y=dataframe["SOC_Cell4"], name = "SOC Cell4", ))
    fig4.update_layout(font=dict(
        family="Times New Roman",
        size=fontsize,  
        color = 'black'),
        title = 'State of charge of cells',   
    )
    fig4.update_xaxes(title_text="Time [sec] ")
    fig4.update_yaxes(title_text="Percent [%]")
    fig4.show()

    fig5 = go.Figure()
    fig5.add_trace(go.Line(x=dataframe["time"], y=dataframe["SOC_est_Cell1"], name = "SOC estimated Cell1", line=dict(color='purple')))
    fig5.add_trace(go.Line(x=dataframe["time"], y=dataframe["SOC_est_Cell2"], name = "SOC estimated Cell2", line=dict(color='hotpink')))
    fig5.add_trace(go.Line(x=dataframe["time"], y=dataframe["SOC_est_Cell3"], name = "SOC estimated Cell3", line=dict(color='darkblue')))
    fig5.add_trace(go.Line(x=dataframe["time"], y=dataframe["SOC_est_Cell4"], name = "SOC estimated Cell4", ))
    fig5.update_layout(font=dict(
        family="Times New Roman",
        size=fontsize,  
        color = 'black'),
        title = 'State of charge of cells estimated by BMS',   
    )
    fig5.update_xaxes(title_text="Time [sec] ")
    fig5.update_yaxes(title_text="Percent [%]")
    fig5.show()

    return 

def plot_robustness_plotly(dataframe, fontsize, output, min, max):

    name = output.replace("_", " " ) 

    col1 = 'purple'
    col2 = 'hotpink'
    col3 = 'blue'
    col4 = '#9467bd'
    robcol  = 'magenta'

    if '1' in output:
        outputcol = col1
    if '2' in output:
        outputcol = col2
    if '3' in output:
        outputcol = col3
    if '4' in output:
        outputcol = col4
    
    fig2 = go.Figure()
    fig2.add_trace(go.Line(x=dataframe["time"], y=dataframe[output], name = name, line=dict(color=outputcol)))
    fig2.add_trace(go.Line(x=dataframe["time"], y=dataframe['Robustness'],  name = "Robustness", line=dict(color=robcol)))
    fig2.update_layout(font=dict(
        family="Times New Roman",
        size=fontsize,  
        color = 'black'),
        title = 'Robustness of voltage of cell 2',   
    )
    fig2.update_xaxes(title_text="Time [sec] ")
    fig2.update_yaxes(title_text="Voltage [V]")
    fig2.add_hrect(y0=max, y1=min, line_width=0, fillcolor="red", opacity=0.2, annotation_text="STL")
    fig2.show()
    return


