from functions import *
from DT_inputs_output import *

df_opt = pd.DataFrame(columns= ['iter', 'target','x1','x2','x3','x4'])

app = dash.Dash(__name__)

app.layout = html.Div([
    html.Div([
        html.Div([
            html.H2("STL"),
            dcc.Dropdown(['SOC_Cell1', 'V_terminal_Cell1', 'I_Cell1', 'SOC_Cell2', 'V_terminal_Cell2', 'I_Cell2', 'SOC_Cell3', 'V_terminal_Cell3', 'I_Cell3', 'SOC_Cell4', 'V_terminal_Cell4', 'I_Cell4', 'T_Cell1', 'SOC_est_Cell1','SOC_est_Cell2', 'SOC_est_Cell3', 'SOC_est_Cell4', 'V_pack', 'V_terminal_Cell3_measured'],
                         'V_terminal_Cell2',
                         clearable = False,
                         id = 'STLdropdown'),
            html.Div(html.P([ html.B()])),
            "Max:",dcc.Input('5', type = 'number', step = 0.01, id = 'STLmax', style = {'width': '15%'}),
            html.Div(html.P([ html.B()])),
            "Min:",dcc.Input('4', type = 'number', step = 0.01, id = 'STLmin', style = {'width': '15%'}),
            html.Div(html.P([ html.Br()])),
            html.Div([],id = 'STL-output')
            
        ],id = 'STL',  style = {'display': 'inline-block'}),

        html.Div([
            html.H2("Bayesian optimization"),
            "Cell_2_R1_T, x1 interval: ", dcc.Input('0.1', type = 'number',step = 0.01, id = 'x3_bottom', style = {'width': '15%'}), ' to ', dcc.Input('2', type = 'number', max = 1000,step = 0.01, id = 'x3_top', style = {'width': '15%'}),
            html.Div(html.P([ html.B()])),
            "Cell_2_R2_T, x2 interval: ", dcc.Input('0.1', type = 'number', step = 0.01,  id = 'x4_bottom', style = {'width': '15%'}), ' to ', dcc.Input('2', type = 'number', max = 1000, step = 0.01, id = 'x4_top', style = {'width': '15%'}),
            html.Div(html.P([ html.B()])),
            "Cell_2_C1_T, x3 interval: ", dcc.Input('0.1', type = 'number', step = 0.01, id = 'x1_bottom', style = {'width': '15%'}), ' to ', dcc.Input('2', type = 'number', max = 1000,step = 0.01, id = 'x1_top', style = {'width': '15%'}),
            html.Div(html.P([ html.B()])),
            "Cell_2_C2_T, x4 interval: ", dcc.Input('0.1', type = 'number', step = 0.01,id = 'x2_bottom', style = {'width': '15%'}), ' to ', dcc.Input('2', type = 'number',max = 1000, step = 0.01, id = 'x2_top', style = {'width': '15%'}),
            
            html.Div(html.P([ html.B()])),
            html.P(["Hyper parameters:"], style = {'font-weight':' bold'}),
            "Exploitation vs exploration: ",dcc.Input('2', type = 'number', step = 0.1, id = 'kappa', style = {'width': '15%'}), "(0.1 - 10)",
            html.Div(html.P([ html.B()])),
            "Number of iterations:", dcc.Input('4', type = 'number', step = 1, id = 'n_iter', style = {'width': '15%'}),
            html.Div(html.P([ html.B()])),
            "Iterations before exploration:", dcc.Input('5', type = 'number', step = 1, id = 'init_points', style = {'width': '15%'}),
            html.Div(html.P([ html.B()])),
            html.Button('Optimize', type = 'submit', id = 'opt_btn'),
            html.Div(html.P([ html.B()])),
        ], id = 'bayesian-optimization'),
        html.Div([],id = 'optimization-output'),
        
    ], id = 'left-container',  style = {'display': 'inline-block' }),
    
    html.Div([
        html.H2('Simulate'),
            dcc.Store(id='store-data', data=[], storage_type='memory'), # 'local' or 'session'
            "Simulation time: ", dcc.Input('20000', type = 'number', id = 'sim_time', style = {'width': '5%'}, step = 1000),
            html.Div(html.P([ html.B()])),
            html.Button('Simulate', type = 'submit', id = 'sim_btn'),
            html.Div([],id = 'simulation-output'),
        ], id = 'center-container', style = {'display': 'inline-block' }),

], id = 'container')   

####################################

@app.callback(
    Output('STL-output', 'children'),
    Input('STLdropdown', 'value'),
    Input('STLmin', 'value'), 
    Input('STLmax', 'value')
)
def update_STL(STLdropdown, STLmin, STLmax):
    return html.Div(["φ1 = {} is less than {}".format(STLdropdown, STLmax),
                     html.Div(html.P([ html.B()])),
                     "φ2 = {} is more than {}".format(STLdropdown, STLmin),
                     html.Div(html.P([ html.B()])),
                     "φ = Always  φ1 And φ2"])

#########################################

@app.callback(
    Output('store-data', 'data'),
    Input('opt_btn', 'n_clicks'),
    State('sim_time','value'), 

    State('STLmin','value'), 
    State('STLmax','value'), 

    State('x1_bottom', 'value'),
    State('x1_top', 'value'),
    State('x2_bottom', 'value'),
    State('x2_top', 'value'),
    State('x3_bottom', 'value'),
    State('x3_top', 'value'),
    State('x4_bottom', 'value'),
    State('x4_top', 'value'),

    State('kappa', 'value'),
    State('init_points', 'value'),
    State('n_iter', 'value'), prevent_initial_callback = True,
)

def optimize(opt_btn, sim_time, STLmin, STLmax, x1_bottom, x1_top, x2_bottom, x2_top, x3_bottom, x3_top, x4_bottom, x4_top, kappa, init_points, n_iter):
    if (sim_time == 0) or (x1_bottom==0) or (x1_top == 0) or (x2_bottom) == 0 or (x2_top == 0) or (x3_bottom == 0) or (x3_top == 0) or (x4_bottom == 0) or (x4_top == 0):
        return dash.no_update
    elif opt_btn:
        start_values_DT2['V_cell3_offset'] = 0 #no sensor drift
        result_org = simulate_fmu('BP_BMS_Passive_Cell_Balancing_SOC_Charging.fmu', start_values=start_values_DT2, output=output_DT2, stop_time=sim_time)  
        df_org = pd.DataFrame(result_org)

        df_org, minval = calculate_robustness(df_org, 'V_terminal_Cell2', float(STLmin), float(STLmax))
        def black_box_function(x1,  x2,  x3,  x4):

            s_key, s_value, keys = sort_start_values(start_values_DT2)
            d = make_dict(keys, s_value)

            d = multiply_by_constant(d, 'Cell_2_R1_T', x1)
            d = multiply_by_constant(d, 'Cell_2_R2_T', x2)
            d = multiply_by_constant(d, 'Cell_2_C1_T', x3)
            d = multiply_by_constant(d, 'Cell_2_C2_T', x4)

            d = make_start_values(d, s_key)

            result = simulate_fmu('BP_BMS_Passive_Cell_Balancing_SOC_Charging.fmu', start_values=d, output=output_DT2, stop_time=sim_time)
            df = pd.DataFrame(result)
    
            df, minval = calculate_robustness(df,'V_terminal_Cell2', float(STLmin), float(STLmax))
            res = minval

            return -res

        pbounds = { 'x1' : (float(x1_bottom), float(x1_top)), 'x2' : (float(x2_bottom), float(x2_top)),  'x3' : (float(x3_bottom), float(x3_top)),
                    'x4' : (float(x4_bottom), float(x4_top))}
        
        optimizer = BayesianOptimization(
            f=black_box_function,
            pbounds=pbounds,
            verbose=2, # verbose = 1 prints only when a maximum is observed, verbose = 0 is silent
            random_state= 1,
        )
        utility = UtilityFunction(kind="ucb", kappa=int(kappa), xi=0.0) 

        optimizer.probe(
            params={'x1': 1, 'x2' :1, 'x3' :1, 'x4' :1},
            lazy=True,
        )

        optimizer.maximize(
            init_points=int(init_points),
            n_iter=int(n_iter),
        )

        data = optimizer.res
        return data
    


#############################################
@app.callback(
    Output('optimization-output', 'children'),
    Input('store-data', 'data'), prevent_initial_callback = True,
)
def utskrift(opt_data):
    if (opt_data is None): #<--- correct the condition
        return dash.no_update
    else: 
        iter_list = []
        target_list = []
        x1_list = []
        x2_list = []
        x3_list = []
        x4_list = []
        for i in range(0, len(opt_data)):
            iter_list.append(i)
            target_list.append(round(opt_data[i]['target'], 3))
            x1_list.append(round(opt_data[i]['params']['x1'], 3))
            x2_list.append(round(opt_data[i]['params']['x2'], 3))
            x3_list.append(round(opt_data[i]['params']['x3'], 3))
            x4_list.append(round(opt_data[i]['params']['x4'], 3))
        
        
        neg_target_list = [ -x for x in target_list]

        df_opt_result = pd.DataFrame(columns= ['iter', 'target','x1','x2','x3','x4'])
        df_opt_result['iter'] = iter_list
        df_opt_result['target'] = neg_target_list
        df_opt_result['x1'] = x1_list
        df_opt_result['x2'] = x2_list
        df_opt_result['x3'] = x3_list
        df_opt_result['x4'] = x4_list
    
    return html.Div([
            dash_table.DataTable(df_opt_result.to_dict('records'), [{"name": i, "id": i} for i in df_opt_result.columns], style_table={'height': '200px', 'width': '400px', 'overflowY': 'auto'},  style_header={
        'color': 'black',
        'fontWeight': 'bold'
        }, style_cell_conditional=[
        {'if': {'column_id': 'iter'},
         'width': '20px'},
        {'if': {'column_id': 'target'},
         'width': '10px'},
        {'if': {'column_id': 'x1'},
         'width': '10px'},
         {'if': {'column_id': 'x2'},
         'width': '10px'},
         {'if': {'column_id': 'x3'},
         'width': '10px'},
         {'if': {'column_id': 'x4'},
         'width': '10px'},
        ], style_data_conditional=[{'if': {'filter_query': '{{target}} = {}'.format(df_opt_result['target'].min())},
                                'color': 'magenta'}]),
        ])

##########################################

@app.callback(
    Output('simulation-output', 'children'), 
    State('store-data', 'data'),
    Input('sim_btn', 'n_clicks'),
    Input('sim_time', 'value'),
    State('STLmin','value'), 
    State('STLmax','value'), 
)

def runsimulation(opt_data, sim_btn, sim_time, STLmin, STLmax):
    if (opt_data is None) or (sim_time == 0): 
        return dash.no_update
    elif sim_btn:
        filename = 'BP_BMS_Passive_Cell_Balancing_SOC_Charging.fmu'
        opt_max = max(opt_data, key=lambda x:x['target'])
        opt_max_params = opt_max['params']
        x1 = opt_max_params.get('x1')
        x2 = opt_max_params.get('x2')
        x3 = opt_max_params.get('x3')
        x4 = opt_max_params.get('x4')
    
        s_key, s_value, keys = sort_start_values(start_values_DT2)
        d = make_dict(keys, s_value)
        d = multiply_by_constant(d, 'Cell_2_R1_T', x1)
        d = multiply_by_constant(d, 'Cell_2_R2_T', x2)
        d = multiply_by_constant(d, 'Cell_2_C1_T', x3)
        d = multiply_by_constant(d, 'Cell_2_C2_T', x4)

        d = make_start_values(d, s_key)

        result = simulate_fmu(filename, start_values=d, output=output_DT2, stop_time=sim_time)

        df = pd.DataFrame(result)
        df, minval = calculate_robustness(df,'V_terminal_Cell2', float(STLmin), float(STLmax))
        
        fig1 = go.Figure()
        fig1.add_trace(go.Line(x=df["time"], y=df["V_terminal_Cell1"],  name = "V terminal Cell1"))
        fig1.add_trace(go.Line(x=df["time"], y=df["V_terminal_Cell2"],  name = "V terminal Cell2"))
        fig1.add_trace(go.Line(x=df["time"], y=df["V_terminal_Cell3"],  name = "V terminal Cell3"))
        fig1.add_trace(go.Line(x=df["time"], y=df["V_terminal_Cell4"],  name = "V terminal Cell4"))
        fig1.update_layout(
            font_family = "Trebuchet MS",
            title_font_family = "Trebuchet MS",
            title = 'Voltage of cells',   
        )
        fig1.update_xaxes(title_text="Time [sec] ")
        fig1.update_yaxes(title_text="Voltage [V]")

        fig2 = go.Figure()
        fig2.add_trace(go.Line(x=df["time"], y=df["V_terminal_Cell2"], name = "V terminal Cell2"))
        fig2.add_trace(go.Line(x=df["time"], y=df['Robustness'],  name = "Robustness"))
        fig2.update_layout(
            font_family = "Trebuchet MS",
            title_font_family = "Trebuchet MS",
            title = 'Robustness of voltage of cell 2',   
        )
        fig2.update_xaxes(title_text="Time [sec] ")
        fig2.update_yaxes(title_text="Voltage [V]")
        fig2.add_hrect(y0=STLmax, y1=STLmin, line_width=0, fillcolor="red", opacity=0.2, annotation_text="STL")
    

        fig3 = go.Figure()
        fig3.add_trace(go.Line(x=df["time"], y=df["I_Cell1"],  name = "I Cell1"))
        fig3.add_trace(go.Line(x=df["time"], y=df["I_Cell2"] , name = "I Cell2"))
        fig3.add_trace(go.Line(x=df["time"], y=df["I_Cell3"], name = "I Cell3"))
        fig3.add_trace(go.Line(x=df["time"], y=df["I_Cell4"], name = "I Cell4"))
        fig3.update_layout(
            font_family = "Trebuchet MS",
            title_font_family = "Trebuchet MS",
            title = 'Current of cells',   
        )
        fig3.update_xaxes(title_text="Time [sec] ")
        fig3.update_yaxes(title_text="Ampere [A]")

        fig4 = go.Figure()
        fig4.add_trace(go.Line(x=df["time"], y=df["SOC_Cell1"], name = "SOC Cell1"))
        fig4.add_trace(go.Line(x=df["time"], y=df["SOC_Cell2"], name = "SOC Cell2"))
        fig4.add_trace(go.Line(x=df["time"], y=df["SOC_Cell3"], name = "SOC Cell3"))
        fig4.add_trace(go.Line(x=df["time"], y=df["SOC_Cell4"], name = "SOC Cell4"))
        fig4.update_layout(
            font_family = "Trebuchet MS",
            title_font_family = "Trebuchet MS",
            title = 'State of charge of cells',   
        )
        fig4.update_xaxes(title_text="Time [sec] ")
        fig4.update_yaxes(title_text="Percent [%]")

        fig5 = go.Figure()
        fig5.add_trace(go.Line(x=df["time"], y=df["SOC_est_Cell1"], name = "SOC estimated Cell1"))
        fig5.add_trace(go.Line(x=df["time"], y=df["SOC_est_Cell2"], name = "SOC estimated Cell2"))
        fig5.add_trace(go.Line(x=df["time"], y=df["SOC_est_Cell3"], name = "SOC estimated Cell3"))
        fig5.add_trace(go.Line(x=df["time"], y=df["SOC_est_Cell4"], name = "SOC estimated Cell4"))
        fig5.update_layout(
            font_family = "Trebuchet MS",
            title_font_family = "Trebuchet MS",
            title = 'State of charge of cells estimated by BMS',   
        )
        fig5.update_xaxes(title_text="Time [sec] ")
        fig5.update_yaxes(title_text="Percent [%]")

        return html.Div([dcc.Graph(id = 'figure1',
                            figure = fig1),
                        dcc.Graph(id = 'figure2',
                            figure = fig2),
                        dcc.Graph(id='figure3', 
                            figure = fig3),
                        dcc.Graph(id='figure4', 
                            figure =  fig4),
                        dcc.Graph(id='figure5', 
                            figure = fig5),])
                  

##################################

if __name__ == "__main__":
    app.run_server(debug=True)

